@@ -1,83 +1,80 @@
-# -*- coding: utf-8 -*-
 # Copyright: See the LICENSE file.
 
 from .base import (
-    Factory,
     BaseDictFactory,
-    DictFactory,
     BaseListFactory,
+    DictFactory,
+    Factory,
     ListFactory,
     StubFactory,
-
     use_strategy,
 )
-
-from .enums import (
-    BUILD_STRATEGY,
-    CREATE_STRATEGY,
-    STUB_STRATEGY,
-)
-
-
-from .errors import (
-    FactoryError,
-)
-
-from .faker import Faker
-
 from .declarations import (
-    LazyFunction,
-    LazyAttribute,
-    Iterator,
-    Sequence,
-    LazyAttributeSequence,
-    SelfAttribute,
-    Trait,
     ContainerAttribute,
-    SubFactory,
     Dict,
+    Iterator,
+    LazyAttribute,
+    LazyAttributeSequence,
+    LazyFunction,
     List,
     Maybe,
     PostGeneration,
     PostGenerationMethodCall,
     RelatedFactory,
     RelatedFactoryList,
+    SelfAttribute,
+    Sequence,
+    SubFactory,
+    Trait,
 )
-
+from .enums import BUILD_STRATEGY, CREATE_STRATEGY, STUB_STRATEGY
+from .errors import FactoryError
+from .faker import Faker
 from .helpers import (
-    debug,
-
     build,
-    create,
-    stub,
-    generate,
-    simple_generate,
-    make_factory,
-
     build_batch,
-    create_batch,
-    stub_batch,
-    generate_batch,
-    simple_generate_batch,
-
-    lazy_attribute,
-    iterator,
-    sequence,
-    lazy_attribute_sequence,
     container_attribute,
+    create,
+    create_batch,
+    debug,
+    generate,
+    generate_batch,
+    iterator,
+    lazy_attribute,
+    lazy_attribute_sequence,
+    make_factory,
     post_generation,
+    sequence,
+    simple_generate,
+    simple_generate_batch,
+    stub,
+    stub_batch,
 )
 
-# Backward compatibility; this should be removed soon.
-from . import alchemy
-from . import django
-from . import mogo
-from . import mongoengine
+try:
+    import alchemy
+except ImportError:
+    pass
+try:
+    import django
+except ImportError:
+    pass
+try:
+    import mogo
+except ImportError:
+    pass
+try:
+    import mongoengine
+except ImportError:
+    pass
 
-
-__version__ = '2.12.0'
 __author__ = 'Raphaël Barrois <raphael.barrois+fboy@polytechnique.org>'
+try:
+    # Python 3.8+
+    from importlib.metadata import version
 
+    __version__ = version("factory_boy")
+except ImportError:
+    import pkg_resources
 
-MogoFactory = mogo.MogoFactory
-DjangoModelFactory = django.DjangoModelFactory
+    __version__ = pkg_resources.get_distribution("factory_boy").version
