@@ -1,31 +1,18 @@
-# -*- coding: utf-8 -*-
 # Copyright: See the LICENSE file.
 
 
 """factory_boy extensions for use with the Django framework."""
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
 
-import io
-import os
-import logging
 import functools
+import io
+import logging
+import os
 
-try:
-    import django
-    from django.core import files as django_files
-    from django.db import IntegrityError
-except ImportError as e:  # pragma: no cover
-    django = None
-    django_files = None
-    import_failure = e
+from django.core import files as django_files
+from django.db import IntegrityError
 
-
-from . import base
-from . import declarations
-from . import errors
-from .compat import is_string
+from . import base, declarations, errors
 
 logger = logging.getLogger('factory.generate')
 
@@ -33,12 +20,6 @@ logger = logging.getLogger('factory.generate')
 DEFAULT_DB_ALIAS = 'default'  # Same as django.db.DEFAULT_DB_ALIAS
 
 
-def require_django():
-    """Simple helper to ensure Django is available."""
-    if django_files is None:  # pragma: no cover
-        raise import_failure
-
-
 _LAZY_LOADS = {}
 
 
@@ -57,24 +38,19 @@ def _lazy_load_get_model():
     get_model loads django.conf.settings, which may fail if
     the settings haven't been configured yet.
     """
-    if django is None:
-        def _get_model(app, model):
-            raise import_failure
-    else:
-        from django import apps as django_apps
-        _get_model = django_apps.apps.get_model
-    _LAZY_LOADS['get_model'] = _get_model
+    from django import apps as django_apps
+    _LAZY_LOADS['get_model'] = django_apps.apps.get_model
 
 
 class DjangoOptions(base.FactoryOptions):
     def _build_default_options(self):
-        return super(DjangoOptions, self)._build_default_options() + [
+        return super()._build_default_options() + [
             base.OptionDefault('django_get_or_create', (), inherit=True),
             base.OptionDefault('database', DEFAULT_DB_ALIAS, inherit=True),
         ]
 
     def _get_counter_reference(self):
-        counter_reference = super(DjangoOptions, self)._get_counter_reference()
+        counter_reference = super()._get_counter_reference()
         if (counter_reference == self.base_factory
                 and self.base_factory._meta.model is not None
                 and self.base_factory._meta.model._meta.abstract
@@ -86,7 +62,7 @@ class DjangoOptions(base.FactoryOptions):
         return counter_reference
 
     def get_model_class(self):
-        if is_string(self.model) and '.' in self.model:
+        if isinstance(self.model, str) and '.' in self.model:
             app, model_name = self.model.split('.', 1)
             self.model = get_model(app, model_name)
 
@@ -110,7 +86,7 @@ class DjangoModelFactory(base.Factory):
     @classmethod
     def _load_model_class(cls, definition):
 
-        if is_string(definition) and '.' in definition:
+        if isinstance(definition, str) and '.' in definition:
             app, model = definition.split('.', 1)
             return get_model(app, model)
 
@@ -138,7 +114,7 @@ class DjangoModelFactory(base.Factory):
         # Original params are used in _get_or_create if it cannot build an
         # object initially due to an IntegrityError being raised
         cls._original_params = params
-        return super(DjangoModelFactory, cls)._generate(strategy, params)
+        return super()._generate(strategy, params)
 
     @classmethod
     def _get_or_create(cls, model_class, *args, **kwargs):
@@ -162,16 +138,21 @@ class DjangoModelFactory(base.Factory):
 
         try:
             instance, _created = manager.get_or_create(*args, **key_fields)
-        except IntegrityError:
-            try:
-                instance = manager.get(**cls._original_params)
-            except manager.model.DoesNotExist:
-                raise ValueError(
-                    "django_get_or_create - Unable to create a new object "
-                    "due an IntegrityError raised based on "
-                    "your model's uniqueness constraints. "
-                    "DoesNotExist: Unable to find an existing object based on "
-                    "the fields specified in your factory instance.")
+        except IntegrityError as e:
+            get_or_create_params = {
+                lookup: value
+                for lookup, value in cls._original_params.items()
+                if lookup in cls._meta.django_get_or_create
+            }
+            if get_or_create_params:
+                try:
+                    instance = manager.get(**get_or_create_params)
+                except manager.model.DoesNotExist:
+                    # Original params are not a valid lookup and triggered a create(),
+                    # that resulted in an IntegrityError. Follow Django’s behavior.
+                    raise e
+            else:
+                raise e
 
         return instance
 
@@ -197,10 +178,6 @@ class FileField(declarations.ParameteredAttribute):
 
     DEFAULT_FILENAME = 'example.dat'
 
-    def __init__(self, **defaults):
-        require_django()
-        super(FileField, self).__init__(**defaults)
-
     def _make_data(self, params):
         """Create data for the field."""
         return params.get('data', b'')
@@ -217,8 +194,8 @@ class FileField(declarations.ParameteredAttribute):
 
         if params.get('from_path'):
             path = params['from_path']
-            f = open(path, 'rb')
-            content = django_files.File(f, name=path)
+            with open(path, 'rb') as f:
+                content = django_files.base.ContentFile(f.read())
 
         elif params.get('from_file'):
             f = params['from_file']
@@ -257,23 +234,21 @@ class ImageField(FileField):
     def _make_data(self, params):
         # ImageField (both django's and factory_boy's) require PIL.
         # Try to import it along one of its known installation paths.
-        try:
-            from PIL import Image
-        except ImportError:
-            import Image
+        from PIL import Image
 
         width = params.get('width', 100)
         height = params.get('height', width)
         color = params.get('color', 'blue')
         image_format = params.get('format', 'JPEG')
+        image_palette = params.get('palette', 'RGB')
 
-        thumb = Image.new('RGB', (width, height), color)
         thumb_io = io.BytesIO()
-        thumb.save(thumb_io, format=image_format)
+        with Image.new(image_palette, (width, height), color) as thumb:
+            thumb.save(thumb_io, format=image_format)
         return thumb_io.getvalue()
 
 
-class mute_signals(object):
+class mute_signals:
     """Temporarily disables and then restores any django signals.
 
     Args:
@@ -327,16 +302,8 @@ class mute_signals(object):
     def __call__(self, callable_obj):
         if isinstance(callable_obj, base.FactoryMetaClass):
             # Retrieve __func__, the *actual* callable object.
-            generate_method = callable_obj._generate.__func__
-
-            @classmethod
-            @functools.wraps(generate_method)
-            def wrapped_generate(*args, **kwargs):
-                # A mute_signals() object is not reentrant; use a copy every time.
-                with self.copy():
-                    return generate_method(*args, **kwargs)
-
-            callable_obj._generate = wrapped_generate
+            callable_obj._create = self.wrap_method(callable_obj._create.__func__)
+            callable_obj._generate = self.wrap_method(callable_obj._generate.__func__)
             return callable_obj
 
         else:
@@ -346,3 +313,12 @@ class mute_signals(object):
                 with self.copy():
                     return callable_obj(*args, **kwargs)
             return wrapper
+
+    def wrap_method(self, method):
+        @classmethod
+        @functools.wraps(method)
+        def wrapped_method(*args, **kwargs):
+            # A mute_signals() object is not reentrant; use a copy every time.
+            with self.copy():
+                return method(*args, **kwargs)
+        return wrapped_method
